/*
 * File    : VWDownloadView.java
 * Created : 29-Jan-2004
 * By      : parg
 * 
 * Azureus - a Java Bittorrent client
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details ( see the LICENSE file ).
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package org.gudy.azureus2.ui.webplugin.remoteui.applet.view;

/**
 * @author parg
 *
 */

import java.text.NumberFormat;
import java.util.*;
import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;

import org.gudy.azureus2.ui.webplugin.remoteui.applet.model.*;
import org.gudy.azureus2.core3.util.*;

public class 
VWDownloadView 
{	
	static NumberFormat	float_format = NumberFormat.getInstance();
	
	static{
		float_format.setMaximumFractionDigits(3);
		float_format.setMinimumFractionDigits(3);
	}

	protected JComponent	component;
	
	protected TableSorter	sorter;
	protected JTable		table;
	
	protected ArrayList	listeners	= new ArrayList();

	public
	VWDownloadView(
		final MDDownloadModel		model )
	{
		sorter = new TableSorter(model);
		
		table = new JTable(sorter);
		
		table.setAutoscrolls( true );
		
		table.setShowHorizontalLines( true );
		
		table.setShowVerticalLines( true );
		
		table.getSelectionModel().addListSelectionListener(
				new ListSelectionListener()
				{
					public void
					valueChanged(
						ListSelectionEvent	ev )
					{
						informCurrentSelection();
					}
				});
		
		// table.setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS );
		
		TableColumnModel cm = table.getColumnModel();
		
		int[]	max_widths = { 30, -1, 60, 90, 80, 80, 50, 50, 80, 60, 70, 50, 60, 50 };
		//int[]	max_widths = { 30, -1, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5, 5 };
		
		for (int i=0;i<max_widths.length;i++){
			
			if ( max_widths[i] != -1 ){
				
				cm.getColumn(i).setMaxWidth(max_widths[i]);
			}
		}
		
		int[]	byte_columns 		= { 2, 3, 8 };
		int[]	bytesec_columns 	= { 9, 10 };
		int[]	centre_columns		= { 6, 7 };
		int[]	rhs_columns			= { 5, 11, 13 };
		
		for (int i=0;i<byte_columns.length;i++){
			
			TableColumn column = cm.getColumn( byte_columns[i]);
						
			column.setCellRenderer(
				new DefaultTableCellRenderer()
				{
					public Component 
					getTableCellRendererComponent(
						JTable		table,
						Object 		o_value,
						boolean 	isSelected,
						boolean 	hasFocus,
						int 		row,
						int 		column )
					{
						long	value = ((Long)o_value).longValue();
	
						String	str = DisplayFormatters.formatByteCountToKiBEtc(value);
						
						JLabel	res = (JLabel)super.getTableCellRendererComponent( table, str, isSelected, hasFocus, row, column );
						
						res.setHorizontalAlignment( JLabel.RIGHT );
						
						if ( column%2==1 ){
							
							// res.setBackground(Color.lightGray);
						}
						
						return( res );
					}
				});
		}
		
		for (int i=0;i<bytesec_columns.length;i++){
			
			TableColumn column = cm.getColumn( bytesec_columns[i]);
			
			column.setCellRenderer(
					new DefaultTableCellRenderer()
					{
						public Component 
						getTableCellRendererComponent(
								JTable		table,
								Object 		o_value,
								boolean 	isSelected,
								boolean 	hasFocus,
								int 		row,
								int 		column )
						{
							long	value = ((Long)o_value).longValue();
							
							String	str = DisplayFormatters.formatByteCountToKiBEtcPerSec(value);
							
							JLabel	res = (JLabel)super.getTableCellRendererComponent( table, str, isSelected, hasFocus, row,column );
							
							res.setHorizontalAlignment( JLabel.RIGHT );
							
							if ( column%2==1 ){
								
								// res.setBackground(Color.lightGray);
							}
							
							return( res );
						}
					});
		}
		
		for (int i=0;i<centre_columns.length;i++){
			
			TableColumn column = cm.getColumn( centre_columns[i]);
			
			column.setCellRenderer(
					new DefaultTableCellRenderer()
					{
						public Component 
						getTableCellRendererComponent(
								JTable		table,
								Object 		o_value,
								boolean 	isSelected,
								boolean 	hasFocus,
								int 		row,
								int 		column )
						{
							JLabel	res = (JLabel)super.getTableCellRendererComponent( table, o_value, isSelected, hasFocus, row,column );
							
							res.setHorizontalAlignment( JLabel.CENTER );						

							return( res );
						}
					});
		}
		
		for (int i=0;i<rhs_columns.length;i++){
			
			TableColumn column = cm.getColumn( rhs_columns[i]);
			
			column.setCellRenderer(
					new DefaultTableCellRenderer()
					{
						public Component 
						getTableCellRendererComponent(
								JTable		table,
								Object 		o_value,
								boolean 	isSelected,
								boolean 	hasFocus,
								int 		row,
								int 		column )
						{
							if ( o_value instanceof String){
								
									// truncate excessive strings to avoid the table being
									// taken over by the column (e.g. status can be very long
									// if the torrent has an error)
								
								String	s_value = (String)o_value;
								
								if ( s_value.length() > 30 ){
									
									o_value = s_value.substring(0,30)+"...";
								}
								
							}else if ( o_value instanceof Float ){
								
								o_value = float_format.format(((Float)o_value).doubleValue());
								
							}

							JLabel	res = (JLabel)super.getTableCellRendererComponent( table, o_value, isSelected, hasFocus, row,column );
							
							res.setHorizontalAlignment( JLabel.RIGHT );
							
							if ( column%2==1 ){
								
								// res.setBackground(Color.lightGray);
							}
							

							return( res );
						}
					});
		}
		
		cm.getColumn(4).setCellRenderer(
				new TableCellRenderer()
				{
					public Component 
					getTableCellRendererComponent(
							JTable		table,
							Object 		o_value,
							boolean 	isSelected,
							boolean 	hasFocus,
							int 		row,
							int 		column )
					{
						int	value = ((Integer)o_value).intValue();

						JProgressBar	pb = new JProgressBar();
						
						pb.setMaximum(100);
						
						pb.setValue(value/10);
						
						pb.setStringPainted(true);
						
						pb.setString(""+((double)value)/10+"%");
						
						return( pb );
					}
				});
		
		cm.getColumn(12).setCellRenderer(
				new DefaultTableCellRenderer()
				{
					public Component 
					getTableCellRendererComponent(
							JTable		table,
							Object 		o_value,
							boolean 	isSelected,
							boolean 	hasFocus,
							int 		row,
							int 		column )
					{
						int	value = ((Integer)o_value).intValue();
						
						JLabel	res;
						
						if ( value < 0x7fffffff ){

							double d = ((double)value)/1000;
							
							res = (JLabel)super.getTableCellRendererComponent( table, new Double(d), isSelected, hasFocus, row,column );
							
						}else{
							res = (JLabel)super.getTableCellRendererComponent( table, Constants.INFINITY_STRING, isSelected, hasFocus, row,column );
							
						}
							
						res.setHorizontalAlignment( JLabel.RIGHT );
						
						if ( column%2==1 ){
							
							// res.setBackground(Color.lightGray);
						}
						
	
						return( res );
					}
				});
		
		model.addTableModelListener(
			new TableModelListener()
			{
				public void
				tableChanged(
					TableModelEvent		ev )
				{
					TableModel	mod = table.getModel();
					
					TableColumnModel	col_mod = table.getColumnModel();
					
					for (int i=0;i<mod.getColumnCount();i++){
						
						TableColumn col = col_mod.getColumn(i);
						
						TableCellRenderer rend = col.getCellRenderer();
						
						if ( rend == null ){
							
							continue;
						}
						
						int	max_width = 0;
						
						for (int j=0;j<mod.getRowCount();j++){
						
							Object data = model.getValueAt( j, table.convertColumnIndexToModel(i));
							
							Component comp = rend.getTableCellRendererComponent( table, data, false, false, j, i );
							
							if ( comp instanceof JLabel ){
								
								JLabel	lab = (JLabel)comp;
								
								int width = SwingUtilities.computeStringWidth(lab.getFontMetrics(lab.getFont()),lab.getText());
								
								if ( width > max_width ){
									
									max_width	= width;
								}
							}
						}
						
						max_width += 8;
						
						if( max_width > 0 && max_width > col.getMaxWidth()){
							
							// System.out.println( "setting max width to" + max_width );
							
							col.setMaxWidth( max_width );
							col.setMinWidth( max_width );
						}
					}
				}
			});
		
	
		table.addMouseListener(
			new MouseAdapter()
			{
				public void 
				mousePressed(
					MouseEvent e )
				{
					if ( 	e.isPopupTrigger() ||
							SwingUtilities.isRightMouseButton( e )){
					
						int row = table.rowAtPoint(e.getPoint());
					
						if (row != -1){
													
							table.getSelectionModel().setSelectionInterval(row, row);
							
							for (int i=0;i<listeners.size();i++){
								
								JPopupMenu menu = ((VWDownloadViewListener)listeners.get(i)).popupRequested(VWDownloadView.this, row);
								
								if ( menu != null ){
									
									menu.show( table, e.getX(), e.getY());
								}
							}
						}
					}
				}
			});
			
		sorter.addMouseListenerToHeaderInTable(table);

		sorter.sortByColumn( 0, true );
		
		JScrollPane scrollpane = new JScrollPane(table);
		
		component	= scrollpane;
	}
	
	public JComponent
	getComponent()
	{
		return( component );
	}
	
	public void
	refresh()
	{
		component.revalidate();
		
		component.repaint();
		
		informCurrentSelection();
	}
	
	protected void
	informCurrentSelection()
	{
		for (int i=0;i<listeners.size();i++){
			
			((VWDownloadViewListener)listeners.get(i)).selectionChanged( this, getSelectedRows());
		}
	}
	
	public int[]
	getSelectedRows()
	{
		return(sorter.mapRows(table.getSelectedRows()));
	}
	
	public void
	setSelectedRows(
		int[]	rows )
	{
		rows = sorter.unmapRows( rows );
		
		int	first = 0x7fffffff;
		
		for (int i=0;i<rows.length;i++){
			
			if ( rows[i] != -1 ){
		
				int	index = rows[i];
				
				if ( index < first ){
					
					first	= index;
				}
				
				table.getSelectionModel().addSelectionInterval(index, index);
			}
		}
		
		if ( first != 0x7fffffff ){
			
			table.scrollRectToVisible(table.getCellRect(first, 0, true ));
		}
	}
	
	public void
	addListener(
		VWDownloadViewListener	l )
	{
		listeners.add( l );
	}
}
